#include <Windows.h>

void PrintErrorMessage(char *szFile, char *szFunction, int nLine);

/* Enabling and Disabling Privileges in C++
   http://msdn.microsoft.com/en-us/library/aa446619%28v=vs.85%29.aspx
*/

BOOL SetPrivilege(
    HANDLE hToken,          // access token handle
    LPCTSTR lpszPrivilege,  // name of privilege to enable/disable
    BOOL bEnablePrivilege   // to enable or disable privilege
    ) 
{
    TOKEN_PRIVILEGES tp;
    LUID luid;

    if ( !LookupPrivilegeValue( 
            NULL,            // lookup privilege on local system
            lpszPrivilege,   // privilege to lookup 
            &luid ) )        // receives LUID of privilege
    {
		PrintErrorMessage(__FILE__, "LookupPrivilegeValue", __LINE__);
//        printf("LookupPrivilegeValue error: %u\n", GetLastError() ); 
        return FALSE; 
    }

    tp.PrivilegeCount = 1;
    tp.Privileges[0].Luid = luid;
    if (bEnablePrivilege)
        tp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
    else
        tp.Privileges[0].Attributes = 0;

    // Enable the privilege or disable all privileges.

    if ( !AdjustTokenPrivileges(
           hToken, 
           FALSE, 
           &tp, 
           sizeof(TOKEN_PRIVILEGES), 
           (PTOKEN_PRIVILEGES) NULL, 
           (PDWORD) NULL) )
    { 
		PrintErrorMessage(__FILE__, "AdjustTokenPrivileges", __LINE__);
//          printf("AdjustTokenPrivileges error: %u\n", GetLastError() ); 
        return FALSE; 
    } 

    if (GetLastError() == ERROR_NOT_ALL_ASSIGNED)

    {
//          printf("The token does not have the specified privilege. \n");
		PrintErrorMessage(__FILE__, "AdjustTokenPrivileges", __LINE__);
          return FALSE;
    } 

    return TRUE;
}